package dev.shadowsoffire.apotheosis.adventure.loot;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.google.common.base.Preconditions;
import com.google.common.base.Predicates;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.apotheosis.adventure.AdventureConfig;
import dev.shadowsoffire.apotheosis.adventure.loot.ShieldBreakerTest;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Predicate;
import javax.annotation.Nullable;
import net.minecraft.util.ExtraCodecs;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.entity.ai.attributes.Attributes;
import net.minecraft.world.item.BlockItem;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraftforge.common.ToolActions;
import net.minecraftforge.registries.ForgeRegistries;

public final class LootCategory {
  private static final Map<String, dev.shadowsoffire.apotheosis.adventure.loot.LootCategory> BY_ID_INTERNAL = new HashMap<>();
  
  private static final List<dev.shadowsoffire.apotheosis.adventure.loot.LootCategory> VALUES_INTERNAL = new LinkedList<>();
  
  public static final Map<String, dev.shadowsoffire.apotheosis.adventure.loot.LootCategory> BY_ID = Collections.unmodifiableMap(BY_ID_INTERNAL);
  
  public static final List<dev.shadowsoffire.apotheosis.adventure.loot.LootCategory> VALUES = Collections.unmodifiableList(VALUES_INTERNAL);
  
  public static final Codec<dev.shadowsoffire.apotheosis.adventure.loot.LootCategory> CODEC = ExtraCodecs.m_184405_(dev.shadowsoffire.apotheosis.adventure.loot.LootCategory::getName, dev.shadowsoffire.apotheosis.adventure.loot.LootCategory::byId);
  
  public static final Codec<Set<dev.shadowsoffire.apotheosis.adventure.loot.LootCategory>> SET_CODEC = PlaceboCodecs.setOf(CODEC);
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory BOW;
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory CROSSBOW;
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory PICKAXE;
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory SHOVEL;
  
  static {
    BOW = register("bow", s -> s.m_41720_() instanceof net.minecraft.world.item.BowItem, arr(new EquipmentSlot[] { EquipmentSlot.MAINHAND, EquipmentSlot.OFFHAND }));
    CROSSBOW = register("crossbow", s -> s.m_41720_() instanceof net.minecraft.world.item.CrossbowItem, arr(new EquipmentSlot[] { EquipmentSlot.MAINHAND, EquipmentSlot.OFFHAND }));
    PICKAXE = register("pickaxe", s -> s.canPerformAction(ToolActions.PICKAXE_DIG), arr(new EquipmentSlot[] { EquipmentSlot.MAINHAND }));
    SHOVEL = register("shovel", s -> s.canPerformAction(ToolActions.SHOVEL_DIG), arr(new EquipmentSlot[] { EquipmentSlot.MAINHAND }));
  }
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory HEAVY_WEAPON = register("heavy_weapon", (Predicate<ItemStack>)new ShieldBreakerTest(), arr(new EquipmentSlot[] { EquipmentSlot.MAINHAND }));
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory HELMET = register("helmet", armorSlot(EquipmentSlot.HEAD), arr(new EquipmentSlot[] { EquipmentSlot.HEAD }));
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory CHESTPLATE = register("chestplate", armorSlot(EquipmentSlot.CHEST), arr(new EquipmentSlot[] { EquipmentSlot.CHEST }));
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory LEGGINGS = register("leggings", armorSlot(EquipmentSlot.LEGS), arr(new EquipmentSlot[] { EquipmentSlot.LEGS }));
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory BOOTS = register("boots", armorSlot(EquipmentSlot.FEET), arr(new EquipmentSlot[] { EquipmentSlot.FEET }));
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory SHIELD;
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory TRIDENT;
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory SWORD;
  
  static {
    SHIELD = register("shield", s -> s.canPerformAction(ToolActions.SHIELD_BLOCK), arr(new EquipmentSlot[] { EquipmentSlot.MAINHAND, EquipmentSlot.OFFHAND }));
    TRIDENT = register("trident", s -> s.m_41720_() instanceof net.minecraft.world.item.TridentItem, arr(new EquipmentSlot[] { EquipmentSlot.MAINHAND }));
    SWORD = register("sword", s -> 
        (s.canPerformAction(ToolActions.SWORD_DIG) || s.m_41720_().getAttributeModifiers(EquipmentSlot.MAINHAND, s).get(Attributes.f_22281_).stream().anyMatch(())), arr(new EquipmentSlot[] { EquipmentSlot.MAINHAND }));
  }
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory NONE = register("none", Predicates.alwaysFalse(), new EquipmentSlot[0]);
  
  private final String name;
  
  private final Predicate<ItemStack> validator;
  
  private final EquipmentSlot[] slots;
  
  private LootCategory(String name, Predicate<ItemStack> validator, EquipmentSlot[] slots) {
    this.name = Preconditions.<String>checkNotNull(name);
    this.validator = Preconditions.<Predicate<ItemStack>>checkNotNull(validator);
    this.slots = Preconditions.<EquipmentSlot[]>checkNotNull(slots);
  }
  
  public String getDescId() {
    return "text.apotheosis.category." + this.name;
  }
  
  public String getDescIdPlural() {
    return getDescId() + ".plural";
  }
  
  public String getName() {
    return this.name;
  }
  
  public EquipmentSlot[] getSlots() {
    return this.slots;
  }
  
  public boolean isValid(ItemStack stack) {
    return this.validator.test(stack);
  }
  
  public boolean isArmor() {
    return (this == HELMET || this == CHESTPLATE || this == LEGGINGS || this == BOOTS);
  }
  
  public boolean isBreaker() {
    return (this == PICKAXE || this == SHOVEL);
  }
  
  public boolean isRanged() {
    return (this == BOW || this == CROSSBOW || this == TRIDENT);
  }
  
  public boolean isDefensive() {
    return (isArmor() || this == SHIELD);
  }
  
  public boolean isLightWeapon() {
    return (this == SWORD || this == TRIDENT);
  }
  
  public boolean isWeapon() {
    return (this == SWORD || this == HEAVY_WEAPON || this == TRIDENT);
  }
  
  public boolean isWeaponOrShield() {
    return (isLightWeapon() || this == SHIELD);
  }
  
  public boolean isNone() {
    return (this == NONE);
  }
  
  public String toString() {
    return String.format("LootCategory[%s]", new Object[] { this.name });
  }
  
  public int hashCode() {
    return this.name.hashCode();
  }
  
  public boolean equals(Object obj) {
    if (obj instanceof dev.shadowsoffire.apotheosis.adventure.loot.LootCategory) {
      dev.shadowsoffire.apotheosis.adventure.loot.LootCategory cat = (dev.shadowsoffire.apotheosis.adventure.loot.LootCategory)obj;
      if (cat.name.equals(this.name));
    } 
    return false;
  }
  
  public static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory register(@Nullable dev.shadowsoffire.apotheosis.adventure.loot.LootCategory orderRef, String name, Predicate<ItemStack> validator, EquipmentSlot[] slots) {
    dev.shadowsoffire.apotheosis.adventure.loot.LootCategory cat = new dev.shadowsoffire.apotheosis.adventure.loot.LootCategory(name, validator, slots);
    if (BY_ID_INTERNAL.containsKey(name))
      throw new IllegalArgumentException("Cannot register a loot category with a duplicate name."); 
    BY_ID_INTERNAL.put(name, cat);
    int idx = VALUES_INTERNAL.size();
    if (orderRef != null)
      idx = VALUES_INTERNAL.indexOf(orderRef); 
    VALUES_INTERNAL.add(idx, cat);
    return cat;
  }
  
  @Nullable
  public static dev.shadowsoffire.apotheosis.adventure.loot.LootCategory byId(String name) {
    return BY_ID.get(name);
  }
  
  public static dev.shadowsoffire.apotheosis.adventure.loot.LootCategory forItem(ItemStack item) {
    if (item.m_41619_())
      return NONE; 
    dev.shadowsoffire.apotheosis.adventure.loot.LootCategory override = (dev.shadowsoffire.apotheosis.adventure.loot.LootCategory)AdventureConfig.TYPE_OVERRIDES.get(ForgeRegistries.ITEMS.getKey(item.m_41720_()));
    if (override != null)
      return override; 
    for (dev.shadowsoffire.apotheosis.adventure.loot.LootCategory c : VALUES) {
      if (c.isValid(item))
        return c; 
    } 
    return NONE;
  }
  
  private static EquipmentSlot[] arr(EquipmentSlot... s) {
    return s;
  }
  
  private static Predicate<ItemStack> armorSlot(EquipmentSlot slot) {
    return stack -> {
        if (!stack.m_150930_(Items.f_42047_)) {
          Item patt8272$temp = stack.m_41720_();
          if (patt8272$temp instanceof BlockItem) {
            BlockItem bi = (BlockItem)patt8272$temp;
            if (bi.m_40614_() instanceof net.minecraft.world.level.block.AbstractSkullBlock)
              return false; 
          } 
        } else {
          return false;
        } 
        return (LivingEntity.m_147233_(stack) == slot);
      };
  }
  
  static final dev.shadowsoffire.apotheosis.adventure.loot.LootCategory register(String name, Predicate<ItemStack> validator, EquipmentSlot[] slots) {
    return register(null, name, validator, slots);
  }
}
